"""
This module contains the global configurations of the framework
"""

#: This name suppose to be used for general meta decoration for functions,
#: methods or even classes
meta_field = '_pyrsmeta'



import threading
import unittest


from .. import context


class TestContext(unittest.TestCase):

    def test_basic_usage(self):
        ctx = context.Context()
        ctx['something'] = 'test'

        self.assertEqual(ctx['something'], 'test')

    def test_context_manager_global_behaviour(self):
        ctx = context.Context({'global': {}})
        ctx['global']['x'] = 1

        with ctx as ictx:
            ictx['global']['x'] = 2

        self.assertEqual(ctx['global'], {'x': 2})

    def test_with_other_thread(self):
        ctx = context.Context()
        ctx['exists'] = 1
        ctx['obj'] = [1, 2, 3]

        def other_thread():
            ctx['obj'] = [1, 2, 3, 4]
            ctx['other'] = 1
            ctx['exists'] = 2
        t = threading.Thread(target=other_thread)
        t.start()
        t.join()
        self.assertNotIn('other', ctx)
        self.assertEqual(ctx['exists'], 1)
        self.assertEqual(ctx['obj'], [1, 2, 3])

    def test_dictlike(self):
        dl = context.Context()
        dl['aaa'] = 12

        self.assertEqual(dict(dl), {'aaa': 12})

    def test_context_manager_copywith(self):
        ctx = context.Context()
        ctx.clear()
        ctx['something'] = 'test'
        ctx['over'] = 'test'
        ctx['obj'] = [1, 2, 3]

        with ctx.copy(justhere='a') as ictx:
            ictx['else'] = 'else'
            ictx['over'] = 'over'
            ictx['obj'] = [1, 2, 3, 4]
            self.assertEqual(ictx['something'], 'test')
            self.assertEqual(ictx['justhere'], 'a')

        self.assertNotIn('else', ctx)
        self.assertNotIn('justhere', ctx)
        self.assertEqual(ctx['obj'], [1, 2, 3])
        self.assertEqual(ctx['something'], 'test')
        self.assertEqual(ctx['over'], 'test')



import unittest

from .. import annotations
from .. import config


class TestMeta(unittest.TestCase):

    def test_ensure_meta(self):
        def func():
            pass

        meta = annotations.ensure_meta(func)

        self.assertIsInstance(meta, dict)
        self.assertEqual(meta, {})

    def test_ensure_meta_with_defined_attrs(self):
        def func():
            pass

        meta = annotations.ensure_meta(func, name='myfunc')

        self.assertIsInstance(meta, dict)
        self.assertEqual(meta, {'name': 'myfunc'})

    def test_get_meta(self):
        def func():
            pass

        annotations.ensure_meta(func, name='myfunc')
        meta = annotations.get_meta(func)

        self.assertIsInstance(meta, dict)
        self.assertEqual(meta, {'name': 'myfunc'})

    def test_get_meta_field(self):
        def func():
            pass

        annotations.ensure_meta(func, name='myfunc')
        field = annotations.get_meta(func, 'name')

        self.assertEqual(field, 'myfunc')

    def test_get_meta_field_default_none(self):
        def func():
            pass

        field = annotations.get_meta(func, 'name')

        self.assertIsNone(field)
        self.assertNotIn('name', annotations.get_meta(func))

    def test_get_meta_field_default_value(self):
        def func():
            pass

        field = annotations.get_meta(func, 'name', 'myfunc')

        self.assertEqual(field, 'myfunc')
        self.assertIn('name', annotations.get_meta(func))
        self.assertEqual(annotations.get_meta(func, 'name'), 'myfunc')

    def test_get_meta_field_default_list(self):
        def func():
            pass

        annotations.get_meta(func, 'list', []).append('12')

        self.assertEqual(annotations.get_meta(func, 'list'), ['12'])


class TestAnnotate(unittest.TestCase):

    def test_annotation_basic(self):
        @annotations.annotate
        def func():
            pass

        self.assertEqual(getattr(func, config['meta_field']), {})

    def test_annotation_with_kwargs(self):
        @annotations.annotate(name='myfunc')
        def func():
            pass

        self.assertEqual(
            getattr(func, config['meta_field']), {'name': 'myfunc'}
        )



"""
Context usage:

.. code:: python

    ctx = Context(value=1)
    with ctx:
        call_a_function(ctx=ctx)

A bit more readable version:

.. code:: python

    ctx = Context(value=1)
    with ctx.copy(value=2, new_value=3) as c:
        call_a_function(ctx=c)
"""
import threading


class Context(threading.local):
    """
    This is a thread safe stack based implementation of context.
    """

    def __init__(self, *args, **kwargs):
        super(Context, self).__init__()
        self.clear(*args, **kwargs)
        self.stack = []

    def __getitem__(self, name):
        return self.data[name]

    def __setitem__(self, name, value):
        self.data[name] = value

    def __iter__(self):
        return self.data.__iter__()

    def __len__(self):
        return len(self.data)

    def keys(self):
        return self.data.keys()

    def get(self, name, default=None):
        return self.data.get(name, default)

    def items(self):
        return self.data.items()

    def copy(self, *args, **kwargs):
        return self.__class__(self.data).update(*args, **kwargs)

    def update(self, *args, **kwargs):
        self.data.update(*args, **kwargs)
        return self

    def clear(self, *args, **kwargs):
        self.data = dict(*args, **kwargs)
        return self

    def _push(self):
        self.stack.append(self.data)
        self.data = self.data.copy()

    def _pop(self):
        self.data = self.stack.pop()

    def __enter__(self):
        self._push()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._pop()


#: pyrs context
ctx = Context()



from .configuration import config


def ensure_meta(func, *args, **kwargs):
    meta = getattr(func, config['meta_field'], None)
    if meta is None:
        meta = {}
        setattr(func, config['meta_field'], meta)
    meta.update(*args, **kwargs)
    return meta


def get_meta(func, name=None, default=None):
    meta = ensure_meta(func)
    if name is None:
        return meta
    if name not in meta and default is not None:
        meta[name] = default
    return meta.get(name)


def annotate(_func=None, **kwargs):
    def decorator(_func):
        ensure_meta(_func, kwargs)
        return _func
    if _func is not None:
        return decorator(_func)
    return decorator



#!/usr/bin/env python
import os

from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


VERSION = '0.4'

if __name__ == '__main__':
    setup(
        name='pyrs',
        author='Csaba Palankai',
        author_email='csaba.palankai@gmail.com',
        packages=find_packages(),
        include_package_data=True,
        version=VERSION,
        license='MIT',
        description="Python microservice framework",
        url='http://pyrs.readthedocs.org/',
        long_description=read('README.rst'),
        classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Natural Language :: English',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Operating System :: OS Independent',
            'Topic :: Internet :: WWW/HTTP :: WSGI',
            'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
            'Topic :: Software Development :: Libraries',
            'Topic :: Software Development :: Libraries :: Python Modules',
        ],
        keywords=('service', 'rest', 'restful', 'swagger', 'resource'),
        zip_safe=False,
        install_requires=[r for r in read("requirements.txt").split("\n") if r],
    )



from .. import conf as _pyrs_config


class Configuration(object):

    def __init__(self, *bases):
        self._conf = dict()
        for base in bases:
            self.upgrade(base)

    def upgrade(self, base):
        for k in dir(base):
            if not k.startswith('_'):
                self._conf[k] = getattr(base, k)

    def __getitem__(self, name):
        return self._conf[name]

    def __iter__(self):
        return self._conf.__iter__()

    def __len__(self):
        return len(self.data)

    def get(self, name, default=None):
        return self._conf.get(name, default)

    def keys(self):
        return self._conf.keys()

    def items(self):
        return self._conf.items()


config = Configuration(_pyrs_config)



import unittest

from .. import configuration


class TestConfigManager(unittest.TestCase):

    def test_basic(self):
        conf = configuration.Configuration()

        with self.assertRaises(TypeError):
            conf['hello'] = 12

    def test_normal(self):
        class Basic(object):
            a = 12

        conf = configuration.Configuration(Basic)
        self.assertEqual(conf['a'], 12)

    def test_multiple(self):
        class A(object):
            a = 1
            b = 2

        class B(object):
            b = 3
            c = 4

        conf = configuration.Configuration(A, B)
        self.assertEqual(conf['a'], 1)
        self.assertEqual(conf['b'], 3)
        self.assertEqual(conf['c'], 4)

    def test_update(self):
        class A(object):
            a = 1
            b = 2

        class B(object):
            b = 3
            c = 4

        conf = configuration.Configuration(A)
        conf.upgrade(B)
        self.assertEqual(conf['a'], 1)
        self.assertEqual(conf['b'], 3)
        self.assertEqual(conf['c'], 4)
